//**************************************************************************
//*                     This file is part of the                           *
//*                MMC - Mpxplay Multimedia Commander                      *
//*                   The source code of MMC is                            *
//*        (C) copyright 1998-2014 by PDSoft (Attila Padar)                *
//*                http://mpxplay.sourceforge.net                          *
//*                  email: mpxplay@freemail.hu                            *
//**************************************************************************
//*  This program is distributed in the hope that it will be useful,       *
//*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
//*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
//*  Please contact with the author (with me) if you want to use           *
//*  or modify this source.                                                *
//**************************************************************************
//function: initialize, load, save gui config

//#define MPXPLAY_USE_DEBUGF 1
#define DISPQT_DEBUG_OUTPUT stdout

#include "mpxplay.h"
#include "control/control.h"
#include "control/cntfuncs.h"
#include "disp_qt.h"
#include "moc_config.h"
#include "moc_mainwindow.h"

struct mmc_dispqt_config_s mmc_config;

static mpxini_var_s mpxpgui_ini[]={
{"MainWinControl", &mmc_config.mainwin_control,(ARG_HEX | ARG_SAVE) },
{"MainWinState",   &mmc_config.mainwin_state,  ARG_SAVE },
{"MainWinPosX",    &mmc_config.mainwin_pos_x,  (ARG_SIGNED|ARG_SAVE) },
{"MainWinPosY",    &mmc_config.mainwin_pos_y,  (ARG_SIGNED|ARG_SAVE) },
{"MainWinSizeX",   &mmc_config.mainwin_size_x, ARG_SAVE },
{"MainWinSizeY",   &mmc_config.mainwin_size_y, ARG_SAVE },
{"MainWinOpGen",   &mmc_config.mainwin_opacity_general, ARG_SAVE },
{"MainWinOpIna",   &mmc_config.mainwin_opacity_inactive, ARG_SAVE },
{"MainWinOpTra",   &mmc_config.mainwin_opacity_transparent, ARG_SAVE },
{"MainWinOpVSh",   &mmc_config.mainwin_opacity_onvideo_show, ARG_SAVE },
{"MainWinOpVHi",   &mmc_config.mainwin_opacity_onvideo_hide, ARG_SAVE },
{"MainWinOpVSKT",  &mmc_config.mainwin_opacity_showkeep_time, ARG_SAVE },
{"MainWinOpVFIT",  &mmc_config.mainwin_opacity_fadein_time, ARG_SAVE },
{"MainWinOpVFOT",  &mmc_config.mainwin_opacity_fadeout_time, ARG_SAVE },
{"EditorControl",  &mmc_config.editor_flags, (ARG_HEX | ARG_SAVE) },
{"EditorSizeY",    &mmc_config.editor_size_y, ARG_SAVE },
{"FontEditor",     &mmc_config.font_text_name[DISPQT_FONTTEXTID_EDITOR],  (ARG_PACHAR|ARG_SAVE) },
{"FontMainwin",    &mmc_config.font_text_name[DISPQT_FONTTEXTID_MAINWIN], (ARG_PACHAR|ARG_SAVE) },
{"FontDialog",     &mmc_config.font_text_name[DISPQT_FONTTEXTID_DIALOG],  (ARG_PACHAR|ARG_SAVE) },
{"ColorEditFontbase",  &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_FONT],  (ARG_PACHAR|ARG_SAVE) },
{"ColorEditBackGr1",   &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_BACK1], (ARG_PACHAR|ARG_SAVE) },
{"ColorEditBackGr2",   &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_BACK2], (ARG_PACHAR|ARG_SAVE) },
{"ColorEditCursor",    &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_CURSOR], (ARG_PACHAR|ARG_SAVE) },
{"ColorEditMark",      &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_MARK],   (ARG_PACHAR|ARG_SAVE) },
{"ColorEditCurrSong",  &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_CURRSONG], (ARG_PACHAR|ARG_SAVE) },
{"ColorEditNextSong",  &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_NEXTSONG], (ARG_PACHAR|ARG_SAVE) },
{"ColorEditPlayed",    &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_PLAYED],   (ARG_PACHAR|ARG_SAVE) },
{"ColorEditNotPlayed", &mmc_config.color_text_name[DISPQT_COLORTEXTID_EDITOR_NOTPLAYED], (ARG_PACHAR|ARG_SAVE) },
{"TransparentControl", &mmc_config.transparent_control, (ARG_HEX | ARG_SAVE) },
{"VideoControl",   &mmc_config.video_control,  (ARG_HEX | ARG_SAVE) },
{NULL, NULL, 0}
};

#define MPXPGUI_INI_CONFIG_LINES (sizeof(mpxpgui_ini) / sizeof(mpxini_var_s))

static const char *config_editortable_colortext_default[DISPQT_COLORTEXTID_EDITOR_MAX] = {
 //"#000000", // DISPQT_COLORTEXTID_EDITOR_FONT : black
 //"#ffffff", // DISPQT_COLORTEXTID_EDITOR_BACK1: white
 //"#f0f0f0", // DISPQT_COLORTEXTID_EDITOR_BACK2: light-gray
 //"#55aaff", // DISPQT_COLORTEXTID_EDITOR_CURSOR: light-blue
 "#aaffff", // DISPQT_COLORTEXTID_EDITOR_FONT : light-cyan
 "#002d4b", // DISPQT_COLORTEXTID_EDITOR_BACK1: dark cyan-blue
 "#002d4b", // DISPQT_COLORTEXTID_EDITOR_BACK2: dark cyan-blue
 "#808080", // DISPQT_COLORTEXTID_EDITOR_CURSOR: gray
 "#ffff00", // DISPQT_COLORTEXTID_EDITOR_MARK:  yellow
 "#00ffff", // DISPQT_COLORTEXTID_EDITOR_CURRSONG: cyan
 "#00ff00", // DISPQT_COLORTEXTID_EDITOR_NEXTSONG: green
 "#c8c8c8", // DISPQT_COLORTEXTID_EDITOR_PLAYED: light-gray
 "#ffffff", // DISPQT_COLORTEXTID_EDITOR_NOTPLAYED: white
};

void mpxplay_dispqt_configini_init(void)
{
	int i;
	struct mmc_dispqt_config_s *gcfg;
	char **color_text_name = &(mmc_config.color_text_name[0]);
	const char **default_colortext = &config_editortable_colortext_default[0];

	memset((void *)&mmc_config, 0, sizeof(mmc_config));

	for(i = 0; i < DISPQT_COLORTEXTID_EDITOR_MAX; i++, color_text_name++, default_colortext++){
		int len = pds_strlen((char *)*default_colortext);
		*color_text_name = new char[len + 4];
		memcpy(*color_text_name, *default_colortext, (len + 1));
	}

	gcfg = &mmc_config;
	gcfg->mainwin_control = DISPQT_CONFIG_MAINWINCTRLS_DEFAULT;
	gcfg->mainwin_pos_x = -1;
	gcfg->mainwin_pos_y = -1;
	gcfg->mainwin_size_x = DISPQT_MAINWIN_INITSIZE_X;
	gcfg->mainwin_size_y = DISPQT_MAINWIN_INITSIZE_Y;
	gcfg->mainwin_opacity_general = DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_DEFAULT;
	gcfg->mainwin_opacity_transparent = DISPQT_CONFIG_MAINWIN_OPACITY_TRANSPARENT_DEFAULT;
	gcfg->mainwin_opacity_inactive = DISPQT_CONFIG_MAINWIN_OPACITY_INACTIVE_DEFAULT;
	gcfg->mainwin_opacity_onvideo_show = DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_SHOW_DEF;
	gcfg->mainwin_opacity_onvideo_hide = DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_HIDE_DEF;
	gcfg->mainwin_opacity_showkeep_time = DISPQT_CONFIG_MAINWIN_OPACITY_SHOWKEEP_TIMEMS_DEFAULT;
	gcfg->mainwin_opacity_fadein_time = DISPQT_CONFIG_MAINWIN_OPACITY_FADEIN_TIMEMS_DEFAULT;
	gcfg->mainwin_opacity_fadeout_time = DISPQT_CONFIG_MAINWIN_OPACITY_FADEOUT_TIMEMS_DEFAULT;
	gcfg->editor_flags = DISPQT_CONFIG_EDITORFLAGS_DEFAULT;
	gcfg->editor_size_y  = 100;
	gcfg->transparent_control = DISPQT_CONFIG_TRANSCTRLS_DEFAULT;
	gcfg->video_control  = DISPQT_CONFIG_VIDEOCTRLS_DEFAULT;
}

void mpxplay_dispqt_config_check(void)
{
	struct mmc_dispqt_config_s *gcfg = &mmc_config;
	if(gcfg->mainwin_opacity_general < DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_MIN)
		gcfg->mainwin_opacity_general = DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_MIN;
	else if(gcfg->mainwin_opacity_general > DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_MAX)
		gcfg->mainwin_opacity_general = DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_MAX;
	if(gcfg->mainwin_opacity_inactive < DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_MIN)
		gcfg->mainwin_opacity_inactive = DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_MIN;
	else if(gcfg->mainwin_opacity_inactive > DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_MAX)
		gcfg->mainwin_opacity_inactive = DISPQT_CONFIG_MAINWIN_OPACITY_GENERAL_MAX;
	if(gcfg->mainwin_opacity_transparent < DISPQT_CONFIG_MAINWIN_OPACITY_TRANSPARENT_MIN)
		gcfg->mainwin_opacity_transparent = DISPQT_CONFIG_MAINWIN_OPACITY_TRANSPARENT_MIN;
	else if(gcfg->mainwin_opacity_transparent > DISPQT_CONFIG_MAINWIN_OPACITY_TRANSPARENT_MAX)
		gcfg->mainwin_opacity_transparent = DISPQT_CONFIG_MAINWIN_OPACITY_TRANSPARENT_MAX;
	if(gcfg->mainwin_opacity_onvideo_show < DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_SHOW_MIN)
		gcfg->mainwin_opacity_onvideo_show = DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_SHOW_MIN;
	else if(gcfg->mainwin_opacity_onvideo_show > DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_SHOW_MAX)
		gcfg->mainwin_opacity_onvideo_show = DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_SHOW_MAX;
	if(gcfg->mainwin_opacity_onvideo_hide < DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_HIDE_MIN)
		gcfg->mainwin_opacity_onvideo_hide = DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_HIDE_MIN;
	else if(gcfg->mainwin_opacity_onvideo_hide > DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_HIDE_MAX)
		gcfg->mainwin_opacity_onvideo_hide = DISPQT_CONFIG_MAINWIN_OPACITY_ONVIDEO_HIDE_MAX;
	if(gcfg->mainwin_opacity_showkeep_time < DISPQT_CONFIG_MAINWIN_OPACITY_SHOWKEEP_TIMEMS_MIN)
		gcfg->mainwin_opacity_showkeep_time = DISPQT_CONFIG_MAINWIN_OPACITY_SHOWKEEP_TIMEMS_MIN;
	else if(gcfg->mainwin_opacity_showkeep_time > DISPQT_CONFIG_MAINWIN_OPACITY_SHOWKEEP_TIMEMS_MAX)
		gcfg->mainwin_opacity_showkeep_time = DISPQT_CONFIG_MAINWIN_OPACITY_SHOWKEEP_TIMEMS_MAX;
	if(gcfg->mainwin_opacity_fadein_time < DISPQT_CONFIG_MAINWIN_OPACITY_FADEIN_TIMEMS_MIN)
		gcfg->mainwin_opacity_fadein_time = DISPQT_CONFIG_MAINWIN_OPACITY_FADEIN_TIMEMS_MIN;
	else if(gcfg->mainwin_opacity_fadein_time > DISPQT_CONFIG_MAINWIN_OPACITY_FADEIN_TIMEMS_MAX)
		gcfg->mainwin_opacity_fadein_time = DISPQT_CONFIG_MAINWIN_OPACITY_FADEIN_TIMEMS_MAX;
	if(gcfg->mainwin_opacity_fadein_time < DISPQT_CONFIG_MAINWIN_OPACITY_FADEOUT_TIMEMS_MIN)
		gcfg->mainwin_opacity_fadein_time = DISPQT_CONFIG_MAINWIN_OPACITY_FADEOUT_TIMEMS_MIN;
	else if(gcfg->mainwin_opacity_fadein_time > DISPQT_CONFIG_MAINWIN_OPACITY_FADEOUT_TIMEMS_MAX)
		gcfg->mainwin_opacity_fadein_time = DISPQT_CONFIG_MAINWIN_OPACITY_FADEOUT_TIMEMS_MAX;
	if(!(gcfg->transparent_control & DISPQT_CONFIG_TRANSCTRL_DESKTOPTRANS_RESTORE))
		funcbit_disable(gcfg->transparent_control, DISPQT_CONFIG_TRANSCTRL_DESKTOPTRANS_ENABLED);
}

struct mmc_dispqt_config_s *mpxplay_dispqt_configini_get(void)
{
	return &mmc_config;
}

void mpxplay_dispqt_configini_load_from_file(void)
{
	FILE *filehand = NULL;
	int linecount;
	char *memfield = NULL;
	mpxini_part_t pt;
	mpxini_line_t mpxgui_config_lines[MPXPGUI_INI_CONFIG_LINES + 8];

	filehand = mpxplay_control_configfile_open((char *)"MPXP_GUI.INI", "r", NULL, NULL);
	if(!filehand)
		return;
	memset(mpxgui_config_lines, 0, sizeof(mpxgui_config_lines));
	linecount = mpxplay_control_configfile_load(filehand, &memfield, mpxgui_config_lines, (MPXPGUI_INI_CONFIG_LINES + 6), 0);
	if(linecount <= 0)
		goto err_out_load;
	pt.filepos = 0;
	pt.partbegin_linenum = 0;
	pt.partlinenum = linecount;
	mpxplay_control_general_loadini(mpxgui_config_lines, &pt, mpxpgui_ini);

	/*{
		mpxini_line_t *cfg = &mpxgui_config_lines[0];
		for(int i = 0; i < MPXPGUI_INI_CONFIG_LINES; i++, cfg++)
			mpxplay_debugf(DISPQT_DEBUG_OUTPUT, "name: \"%s\" val:\"%s\" ", cfg->varnamep, cfg->valuep);
	}*/
	mpxplay_dispqt_config_check();

err_out_load:
	if(filehand)
		fclose(filehand);
	if(memfield)
		free(memfield);
}

void mpxplay_dispqt_configini_save_to_file(void)
{
	FILE *filehand;
	mpxini_part_t pt;
	mpxini_line_t mpxgui_config_lines[MPXPGUI_INI_CONFIG_LINES + 4];

	filehand = mpxplay_control_configfile_open((char *)"MPXP_GUI.INI", "w", NULL, NULL);
	if(!filehand) {
		mpxplay_debugf(DISPQT_DEBUG_OUTPUT, "configsave file open failed!");
		return;
	}
	memset(mpxgui_config_lines, 0, sizeof(mpxgui_config_lines));
	mpxini_var_s *varp = &mpxpgui_ini[0];
	mpxini_line_t *cfg = &mpxgui_config_lines[0];
	for(int i = 0; i < MPXPGUI_INI_CONFIG_LINES; i++, varp++, cfg++){
		cfg->storepoint = varp;
		cfg->varnamep = (char *)varp->name;
		cfg->valuep = (char *)varp->c;
	}
	pt.filepos = 0;
	pt.partbegin_linenum = 0;
	pt.partlinenum = MPXPGUI_INI_CONFIG_LINES;
	mpxplay_control_general_saveini(mpxgui_config_lines, &pt, filehand);
	fclose(filehand);
}
