#ifndef MOC_VIDEO_QT_H
#define MOC_VIDEO_QT_H
 
#include <QWidget>
#include <QtWidgets>
#include <QtMultimedia/QMediaPlayer>
#include <QtMultimediaWidgets/QVideoWidget>
#include "disp_qt.h"

QT_BEGIN_NAMESPACE
class MainWindow;
QT_END_NAMESPACE

#define DISPQT_VIDEO_OPEN_TIMEOUT  5000
#define DISPQT_VIDEO_MUTEX_TIMEOUT DISPQT_MUTEX_TIMEOUT  // 5000

#define DISPQT_VIDEO_CURSORHIDE_TIMEOUT 3000

#define DISPQT_VIDEOWIDGET_SINGLECLICK_TIMEOUTMS 300
#define DISPQT_VIDEOWIDGET_DOUBLECLICK_TIMEOUTMS 300

#define DISPQT_VIDEOWIDGET_CMENUHIDE_TIMEOUTMS 100

class DispQtVideoWidget : public QVideoWidget
{
Q_OBJECT
public:
    DispQtVideoWidget(MainWindow *main_window = 0, QWidget *parent = 0);
    void widget_control(unsigned long mainwin_control);
    bool get_mousecursor_forbidden_hide(void) { return this->mouse_cursor_forbidden_hide; }
    void mouse_cursor_show_timed(bool timed);
    void mouse_cursor_forbid_hide(bool forbid);
    QMenu *video_surface_contextmenu;

private Q_SLOTS:
	void mouse_cursor_hide(void);
	void mouse_singleclick_timeouthandler(void);
	void cmenu_about_hide(void) { this->timer_cmenu_hide.start(DISPQT_VIDEOWIDGET_CMENUHIDE_TIMEOUTMS); this->mouse_cursor_show_timed(true);}
	void play_start_or_pause(void);

protected:
    void keyPressEvent(QKeyEvent *event);
    void mouseDoubleClickEvent(QMouseEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void wheelEvent(QWheelEvent* event);
    void mouseMoveEvent(QMouseEvent *event);
    bool event(QEvent *event);

private:
    MainWindow *main_window;
    QWidget *parent_widget;
    bool mouse_cursor_forbidden_hide, singleclick_event, singleclick_fullsrc_start, fullsrc_curr, is_cursor_blank;
    QTimer *timer_singleclick, timer_doubleclick, *timer_mouse_hide, timer_cmenu_hide;
    QMutex mutex_video;
    QAction *menuaction_filename, *action_play_start, *action_play_stop;
    QAction *action_play_skipback, *action_play_skipforward, *action_view_videofs, *action_file_exit;
};

class DispQtVideoPlayer : public QWidget
{
Q_OBJECT
public:
	DispQtVideoPlayer(MainWindow *main_window = 0, QWidget *parent = 0);
	~DispQtVideoPlayer();
	void video_mediaplayer_open(void);
	void video_mediaplayer_close(void);
	void set_focus(void) { if(this->video_widget) this->video_widget->setFocus(Qt::MouseFocusReason); }
	void video_request_open(void *handle_ptr, char *utf8_filename);
	bool video_wait_for_open(void);
	void video_request_close(void *handle_ptr);
	bool video_wait_for_close(void);
	bool video_is_fullscreen(void) { return ((this->video_widget)? this->video_widget->isFullScreen() : false); }
	qint64 video_tell_ms(void);
	qint64 video_duration_ms(void);
	DispQtVideoWidget *video_get_widgetptr(void) { return this->video_widget; }
	void video_reset_widget(void);
	void video_mouse_cursor_show(bool timed);
	void video_mouse_cursor_forbid_hide(bool forbid);
	//QString video_media_name(void) { return this->media_name; }

Q_SIGNALS:
	void signal_video_open(void);
	void signal_video_close(void);
	void signal_video_play(bool startflag);
	void signal_video_pause(void);
	void signal_video_switch_play(void);
	void signal_video_seek_ms(qint64 pos_ms);
	void signal_video_set_fullscreen(bool fs);
	void signal_video_switch_fullscreen(void);
	void signal_video_apply_fullscreen(bool fs); // 2nd step of switch_fullscreen
	void signal_video_check_windows(void);
	void signal_video_apply_windows(void);       // 3rd step of switch_fullscreen
	void signal_video_event_stop(void);
	void signal_video_set_speed(int speed1000);
	void signal_audio_set_volume(int vol);

private Q_SLOTS:
	void video_open(void);
	void video_close(void);
	void video_play(bool startflag);
	void video_pause(void);
	void video_switch_play(void) { if(this->is_playing) this->video_pause(); else this->video_play(false); }
	void video_stop(void *handle_ptr, bool close_media);
	void video_seek_ms(qint64 pos_ms);
	void video_set_fullscreen(bool fs);
	void video_switch_fullscreen(void);
	void video_apply_fullscreen(bool fs);
	void video_check_windows(void);
	void video_apply_windows(void);
	void video_handle_event_stop(void);
	void video_set_speed(int speed1000);
	void audio_set_volume(int vol);

	void status_changed(QMediaPlayer::MediaStatus status);
	void display_error_message(QMediaPlayer::Error);

protected:
	void closeEvent(QCloseEvent *e);

private:
	void video_cursor_select(QMediaPlayer::MediaStatus status);

	MainWindow *main_window;
	QWidget *parent_widget;
	QHBoxLayout *video_layout;
	QMediaPlayer *qt_media_player;
	DispQtVideoWidget *video_widget;
	QString media_name;
	QUrl media_url;
	QMediaContent media_content;
	bool is_video_show, is_video_open, is_wait_for_open, is_playing;
	bool is_wait_for_close, is_video_closed;
	void *ptr_handle_curr, *ptr_handle_new, *ptr_handle_close; // for close identification
	QMutex mutex_player, mutex_qmedia;
};

#endif
